@@ -104,6 +104,7 @@ class PerturbFlow(nn.Module):
         #self.use_bias = not zero_bias
         
         self.codebook_weights = None
+        self.total_count = None
 
         set_random_seed(seed)
         self.setup_networks()
@@ -317,6 +318,7 @@ class PerturbFlow(nn.Module):
         if self.loss_func=='negbinomial':
             total_count = pyro.param("inverse_dispersion", self.inverse_dispersion *
                                      xs.new_ones(self.input_size), constraint=constraints.positive)
+            self.total_count = total_count
             
         if self.use_zeroinflate:
             gate_logits = pyro.param("dropout_rate", xs.new_zeros(self.input_size))
@@ -396,6 +398,7 @@ class PerturbFlow(nn.Module):
         if self.loss_func=='negbinomial':
             total_count = pyro.param("inverse_dispersion", self.inverse_dispersion *
                                      xs.new_ones(self.input_size), constraint=constraints.positive)
+            self.total_count = total_count
             
         if self.use_zeroinflate:
             gate_logits = pyro.param("dropout_rate", xs.new_zeros(self.input_size))
@@ -480,6 +483,7 @@ class PerturbFlow(nn.Module):
         if self.loss_func=='negbinomial':
             total_count = pyro.param("inverse_dispersion", self.inverse_dispersion *
                                      xs.new_ones(self.input_size), constraint=constraints.positive)
+            self.total_count = total_count
             
         if self.use_zeroinflate:
             gate_logits = pyro.param("dropout_rate", xs.new_zeros(self.input_size))
@@ -576,6 +580,7 @@ class PerturbFlow(nn.Module):
         if self.loss_func=='negbinomial':
             total_count = pyro.param("inverse_dispersion", self.inverse_dispersion *
                                      xs.new_ones(self.input_size), constraint=constraints.positive)
+            self.total_count = total_count
             
         if self.use_zeroinflate:
             gate_logits = pyro.param("dropout_rate", xs.new_zeros(self.input_size))
@@ -876,24 +881,11 @@ class PerturbFlow(nn.Module):
         if self.loss_func == 'bernoulli':
             #counts = self.sigmoid(concentrate)
             counts = dist.Bernoulli(logits=concentrate).to_event(1).mean
-        elif self.loss_func == 'negbinomial':
-            rate = concentrate.exp()
-            theta = dist.DirichletMultinomial(total_count=1, concentration=rate).mean
-            
-            total_count = pyro.param("inverse_dispersion")
-            #store = pyro.get_param_store()
-            #total_count = store['inverse_dispersion']
-            counts = dist.NegativeBinomial(total_count=total_count, probs=theta).to_event(1).mean
-        elif self.loss_func == 'poisson':
+        else:
             rate = concentrate.exp()
             theta = dist.DirichletMultinomial(total_count=1, concentration=rate).mean
             counts = theta * library_size
             #counts = dist.Poisson(rate=rate).to_event(1).mean
-        elif self.loss_func == 'multinomial':
-            rate = concentrate.exp()
-            theta = dist.DirichletMultinomial(total_count=1, concentration=rate).mean
-            #counts = dist.Multinomial(total_count=int(1e8), probs=theta).mean
-            counts = theta * library_size
         return counts
     
     def _count_sample(self,concentrate):
@@ -905,22 +897,17 @@ class PerturbFlow(nn.Module):
             counts = dist.Poisson(rate=counts).to_event(1).sample()
         return counts
     
-    def get_counts(self, zs, library_sizes = None,
+    def get_counts(self, zs, library_sizes,
                         batch_size: int = 1024, 
                         use_sampler: bool = False):
 
         zs = convert_to_tensor(zs, device=self.get_device())
         
-        if self.loss_func in ['multinomial','poisson']:
-            assert library_sizes is not None, 'Library sizes are required for multinomial!'
-            
-            if type(library_sizes) == list:
-                library_sizes = np.array(library_sizes).view(-1,1)
-            elif len(library_sizes.shape)==1:
-                library_sizes = library_sizes.view(-1,1)
-            ls = convert_to_tensor(library_sizes, device=self.get_device())
-        else:
-            ls = zs
+        if type(library_sizes) == list:
+            library_sizes = np.array(library_sizes).view(-1,1)
+        elif len(library_sizes.shape)==1:
+            library_sizes = library_sizes.view(-1,1)
+        ls = convert_to_tensor(library_sizes, device=self.get_device())
             
         dataset = CustomDataset2(zs,ls)
         dataloader = DataLoader(dataset, batch_size=batch_size, shuffle=False)
@@ -1077,6 +1064,9 @@ class PerturbFlow(nn.Module):
                 # Update progress bar
                 pbar.set_postfix({'loss': str_loss})
                 pbar.update(1)
+                
+        if self.loss_func == 'negbinomial':
+            self.total_count = pyro.param('inverse_dispersion')
 
     @classmethod
     def save_model(cls, model, file_path, compression=False):
